define([
    "../../widgets/_ComplexValueWidget",
    "../classes/categories",
    "../classes/topics",
    "./CategoryListView",
    "dijit/form/Button",
    "dijit/form/ComboBox",
    "dojo/_base/declare",
    "dojo/_base/lang",
    "dojo/dnd/Source",
    "dojo/dom-construct",
    "dojo/store/Memory",
    "require",
    "dojo/text!./templates/QuestionEditorToolkit.html"
], function(_ComplexValueWidget, categories, topics, CategoryListView, Button, ComboBox, declare, lang, Source, domConstruct, Memory, require, template) {
    return declare([_ComplexValueWidget], {

        contextRequire: require,
        templateString: template,
        _categoryList: null,
        _categorySelect: null,
        _categoryStore: null,
        _topicSelect: null,
        _topicStore: null,

        _contentItems: [
            { type: "Header" },
            { type: "Text" },
            { type: "Divider" }
        ],
        _inputsItems: [
            { type: "StringInput" },
            { type: "TextInput" },
            { type: "NumberInput" },
            { type: "ScaleInput" },
            { type: "SingleChoiceInput" },
            { type: "MultipleChoiceInput" }
        ],
        _labelMap: {
            "Header": "Header",
            "Text": "Text",
            "Divider": "Divider",
            "StringInput": "Text line",
            "TextInput": "Free text",
            "NumberInput": "Number",
            "ScaleInput": "Scale",
            "SingleChoiceInput": "Single choice",
            "MultipleChoiceInput": "Multiple choice"
        },
        _iconMap: {
            "Header": "Header",
            "Text": "TextBox",
            "Divider": "Divider",
            "StringInput": "Text",
            "TextInput": "Text",
            "NumberInput": "Number",
            "ScaleInput": "Scale",
            "SingleChoiceInput": "MultipleChoice",
            "MultipleChoiceInput": "MultipleChoice"
        },

        buildRendering: function() {
            this.inherited(arguments);

            this._categoryList = new CategoryListView({
                name: "categories"
            }).placeAt(this.categoryListNode);

            this._categoryStore = new Memory({data: [] });
            this._categorySelect = new ComboBox( {
                store: this._categoryStore,
                searchAttr: "id"
            }, this.categoryBoxNode);

            this._topicStore = new Memory( {data: [] });
            this._topicSelect = new ComboBox({
                name: "topic",
                store: this._topicStore,
                searchAttr: "id"
            }, this.topicBoxNode);

            var creator = lang.hitch(this,"_creator");
            this.contentSource = new Source(this.ToolkitContentSourceNode, {
                accept: [],
                copyOnly: true,
                selfAccept: false,
                singular: true,
                creator: creator
            });
            this.inputsSource = new Source(this.ToolkitInputsSourceNode, {
                accept: [],
                copyOnly: true,
                selfAccept: false,
                singular: true,
                creator: creator
            });
            this.contentSource.insertNodes(false, this._contentItems);
            this.inputsSource.insertNodes(false, this._inputsItems);
        },
        startup: function() {
            if ( this._started ) { return; }
            this.inherited(arguments);

            categories.query().forEach(lang.hitch(this,function(cat){
                this._categoryStore.put({ id: cat });
            }));

            topics.query().forEach(lang.hitch(this,function(topic){
                this._categoryStore.put({ id: topic });
            }));
        },
        _onCategoryAdd: function() {
            // it seems, the button is not disabled properly and still
            // generates click events, that's why we chack here.
            if ( this.disabled || this.readOnly ) { return; }
            this._categoryList.appendItem(this._categorySelect.get('displayedValue')); 
            this._categorySelect.reset();
        },
        _creator: function(item, hint) {
            var node;
            if (hint === "avatar") {
                node = domConstruct.create('span',{
                    innerHTML: this._labelMap[item.type] || item.type
                });
            } else {
                var w = new Button({
                    baseClass: "rftLargeButton",
                    iconClass: "rftIcon rftIcon"+(this._iconMap[item.type]||"Unknown"),
                    label: this._labelMap[item.type] || item.type,
                    showLabel: true,
                    style: "display: block"
                });
                node = w.domNode;
            }
            return {
                node: node,
                data: item,
                type: ["questionContent"]
            };
        }

    });
});
